/*
 * Preferences for the extension which will be available in the 
 * "gnome-shell-extension-prefs" tool.
 *
 *
 * see: https://live.gnome.org/GnomeShell/Extensions#Extension_Preferences
 *
 */

//***// imports:

// main
const Gio  = imports.gi.Gio;
const Gtk  = imports.gi.Gtk;
const Lang = imports.lang;
const Me   = imports.misc.extensionUtils.getCurrentExtension();


// define global variables
let settings = {};





//***// basic preferences functions

function init() {

	// load the settings schema
    	settings = _getSettingsSchema();

	// create custom command bindings for gsettings "bool" and "string"
    	let set_boolean = Lang.bind(settings, settings.set_boolean);
    	let set_string = Lang.bind(settings, settings.set_string);

    	settings.set_boolean = function(key, value) {
        	set_boolean(key, value);
        	Gio.Settings.sync();
    	};

    	settings.set_string = function(key, value) {
        	set_string(key, value);
        	Gio.Settings.sync();
    	};

}



function buildPrefsWidget() {
// build the Gtk preferences widget
	let frame = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL, border_width: 10, margin: 10});

	// add items to the widget frame
	frame.add( _createSwitchbox("Show Overview when closing Window on last Workspace", 
		"Opens the Overview even if the current Workspace is the last one", _getOnLastWS(), _setOnLastWS ));

	frame.add( _createSwitchbox("Ignore minimized Windows", 
		"Opens the Overview even if there are minimized windows on the current Workspace", _getIgnoreMinimized(), _setIgnoreMinimized ));

	frame.show_all();
	return frame;
}






//***// preferences functions

function _createSwitchbox(text, tooltip, getFunction, setFunction) {
// create box with text entry for Server IP
	let box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin: 5});
	let label = new Gtk.Label({ label: text, xalign: 0, tooltip_text: tooltip });
	let toggleswitch = new Gtk.Switch({ active: getFunction });

	// connect to "toggled" emit signal
	toggleswitch.connect('notify::active', setFunction);

	//fill the box with content
	box.pack_start(label, true, true, 0);
	box.add(toggleswitch);

	return box;
}






function _getSettingsSchema() {
// get settings schema
    	const GioSSS = Gio.SettingsSchemaSource;

	// get the gsettings schema and path
    	let schema = Me.metadata['settings-schema'];
    	let source = GioSSS.new_from_directory(Me.dir.get_child('schemas').get_path(),
                                           GioSSS.get_default(),
                                           false);


	return new Gio.Settings({
        	settings_schema : source.lookup(schema, true)
    	});

}






// functions to get/set gsettings entries
function _getOnLastWS() {
	return settings.get_boolean('onlastworkspace');
}

function _setOnLastWS() {
    settings.set_boolean('onlastworkspace', !_getOnLastWS());
}


function _getIgnoreMinimized() {
	return settings.get_boolean('ignoreminimized');
}

function _setIgnoreMinimized() {
    settings.set_boolean('ignoreminimized', !_getIgnoreMinimized());
}


